@@ -12,7 +12,6 @@ from pytype import convert_structural
 from pytype import debug
 from pytype import function
 from pytype import metrics
-from pytype import output
 from pytype import special_builtins
 from pytype import state as frame_state
 from pytype import vm
@@ -230,7 +229,7 @@ class CallTracer(vm.VirtualMachine):
       node2.ConnectTo(node0)
     return node0
 
-  def bind_method(self, node, name, methodvar, instance_var):
+  def bind_method(self, node, methodvar, instance_var):
     bound = self.program.NewVariable()
     for m in methodvar.Data(node):
       if isinstance(m, special_builtins.ClassMethodInstance):
@@ -336,7 +335,7 @@ class CallTracer(vm.VirtualMachine):
         node, binding.data.get_class(), method_name, binding)
     if method:
       bound_method = self.bind_method(
-          node, method_name, method, binding.AssignToNewVariable())
+          node, method, binding.AssignToNewVariable())
       node = self.analyze_method_var(node, method_name, bound_method)
     return node
 
@@ -389,7 +388,7 @@ class CallTracer(vm.VirtualMachine):
           for m in (v.fget, v.fset, v.fdel):
             if m:
               methods.insert(0, (name, m))
-      b = self.bind_method(node, name, methodvar, instance)
+      b = self.bind_method(node, methodvar, instance)
       node = self.analyze_method_var(node, name, b, val)
     return node
 
@@ -440,6 +439,8 @@ class CallTracer(vm.VirtualMachine):
       for value in f.bindings:
         if self._should_analyze_as_interpreter_function(value.data):
           node = self.analyze_function(node, value)
+    for func, opcode in self.functions_type_params_check:
+      func.signature.check_type_parameter_count(self.simple_stack(opcode))
     return node
 
   def analyze(self, node, defs, maximum_depth):
@@ -502,7 +503,7 @@ class CallTracer(vm.VirtualMachine):
       annotated_names.add(name)
       data.append(pytd.Constant(name, t))
     for name, var in defs.items():
-      if (name in output.TOP_LEVEL_IGNORE or name in annotated_names or
+      if (name in abstract_utils.TOP_LEVEL_IGNORE or name in annotated_names or
           self._is_typing_member(name, var)):
         continue
       options = var.FilteredData(self.exitpoint, strict=False)
@@ -627,13 +628,7 @@ class CallTracer(vm.VirtualMachine):
   def _check_return(self, node, actual, formal):
     if not self.options.report_errors:
       return True
-    views = abstract_utils.get_views([actual], node)
-    # Check for typevars in the return value first, since bad_matches
-    # expects not to get any.
-    bad = [view for view in views
-           if actual in view and view[actual].data.formal]
-    if not bad:
-      bad = self.matcher.bad_matches(actual, formal, node)
+    bad = self.matcher(node).bad_matches(actual, formal)
     if bad:
       self.errorlog.bad_return_type(
           self.frames, node, formal, actual, bad)
