@@ -164,11 +164,6 @@ def add_basic_options(o):
       dest="bind_properties", default=False,
       help=("Bind @property methods to the classes they're defined on for more "
             "precise type-checking. " + temporary))
-  o.add_argument(
-      "--check-parameter-types", action="store_true",
-      dest="check_parameter_types", default=True,
-      help=("Check parameter defaults and assignments against their "
-            "annotations. " + temporary))
   o.add_argument(
       "--preserve-union-macros", action="store_true",
       dest="preserve_union_macros", default=False,
@@ -177,6 +172,13 @@ def add_basic_options(o):
       "--use-enum-overlay", action="store_true",
       dest="use_enum_overlay", default=False,
       help="Use the enum overlay for more precise enum checking. " + temporary)
+  o.add_argument(
+      "--enforce-noniterable-strings",
+      action="store_true",
+      dest="enforce_noniterable_strings",
+      default=False,
+      help=("Enforces type errors when attempting to pass a `str` into an "
+            "`Iterable` type."))
 
 
 def add_subtools(o):
