@@ -73,6 +73,7 @@ class PythonConstant(metaclass=MixinMeta):
   def init_mixin(self, pyval):
     """Mix-in equivalent of __init__."""
     self.pyval = pyval
+    self._printing = False
 
   def str_of_constant(self, printer):
     """Get a string representation of this constant.
@@ -88,7 +89,13 @@ class PythonConstant(metaclass=MixinMeta):
     return repr(self.pyval)
 
   def __repr__(self):
-    return "<%s %r>" % (self.name, self.str_of_constant(str))
+    if self._printing:  # recursion detected
+      const = "[...]"
+    else:
+      self._printing = True
+      const = self.str_of_constant(str)
+      self._printing = False
+    return "<%s %r>" % (self.name, const)
 
 
 class HasSlots(metaclass=MixinMeta):
@@ -114,8 +121,12 @@ class HasSlots(metaclass=MixinMeta):
   def set_slot(self, name, method):
     """Add a new slot to this value."""
     assert name not in self._slots, "slot %s already occupied" % name
+    # For getting a slot value, we don't need a ParameterizedClass's type
+    # parameters, and evaluating them in the middle of constructing the class
+    # can trigger a recursion error, so use only the base class.
+    base = self.base_cls if self.isinstance_ParameterizedClass() else self
     _, attr = self.vm.attribute_handler.get_attribute(
-        self.vm.root_node, self, name, self.to_binding(self.vm.root_node))
+        self.vm.root_node, base, name, base.to_binding(self.vm.root_node))
     self._super[name] = attr
     f = self.make_native_function(name, method)
     self._slots[name] = f.to_variable(self.vm.root_node)
@@ -183,12 +194,12 @@ class LazyMembers(metaclass=MixinMeta):
   def init_mixin(self, member_map):
     self._member_map = member_map
 
-  def _convert_member(self, member):
+  def _convert_member(self, member, subst=None):
     raise NotImplementedError()
 
-  def load_lazy_attribute(self, name):
+  def load_lazy_attribute(self, name, subst=None):
     """Load the named attribute into self.members."""
     if name not in self.members and name in self._member_map:
-      variable = self._convert_member(self._member_map[name])
+      variable = self._convert_member(self._member_map[name], subst)
       assert isinstance(variable, cfg.Variable)
       self.members[name] = variable
