@@ -4,6 +4,7 @@ import os
 import sys
 
 from pytype import pytype_source_utils
+from pytype import utils
 
 import unittest
 
@@ -12,13 +13,13 @@ class PytypeSourceUtilsTest(unittest.TestCase):
   """Test pytype source utilities."""
 
   def setUp(self):
-    super(PytypeSourceUtilsTest, self).setUp()
+    super().setUp()
     self.root = os.path.dirname(__file__)
 
-  def testPytypeSourceDir(self):
+  def test_pytype_source_dir(self):
     self.assertEqual(self.root, pytype_source_utils.pytype_source_dir())
 
-  def testGetFullPath(self):
+  def test_get_full_path(self):
     self.assertEqual(
         os.path.join(self.root, "foo/bar"),
         pytype_source_utils.get_full_path("foo/bar"))
@@ -26,19 +27,21 @@ class PytypeSourceUtilsTest(unittest.TestCase):
         "/foo/bar",
         pytype_source_utils.get_full_path("/foo/bar"))
 
-  def testListPytypeFiles(self):
-    l = list(pytype_source_utils.list_pytype_files("pytd/stdlib/2"))
+  def test_list_pytype_files(self):
+    l = list(pytype_source_utils.list_pytype_files("stubs/stdlib/2"))
     self.assertIn("_ctypes.pytd", l)
     self.assertIn("collections.pytd", l)
 
-  def testGetCustomPythonExe27(self):
+  def test_get_custom_python_exe27(self):
     exe = pytype_source_utils.get_custom_python_exe((2, 7))
-    self.assertIn("2.7", exe)
+    if utils.USE_ANNOTATIONS_BACKPORT:
+      self.assertIn("2.7", exe)
+    else:
+      self.assertIsNone(exe)
 
-  @unittest.skipUnless(sys.version_info.major == 3, "requires Python 3")
-  def testGetCustomPythonExe3(self):
+  def test_get_custom_python_exe3(self):
     exe = pytype_source_utils.get_custom_python_exe(sys.version_info[:2])
-    self.assertIn("python3", exe)
+    self.assertIsNone(exe)
 
 
 if __name__ == "__main__":
