@@ -51,7 +51,6 @@ class TypeNew(abstract.PyTDFunction):
       return node, abc.members["ABCMeta"].AssignToNewVariable(node)
     node, raw_ret = super().call(node, func, args)
     # Removes TypeVars from the return value.
-    # See test_typevar.TypeVarTest.test_type_of_typevar(_error).
     ret = self.vm.program.NewVariable()
     for b in raw_ret.bindings:
       value = self.vm.annotations_util.deformalize(b.data)
@@ -290,7 +289,7 @@ def _check_against_mro(vm, target, class_spec):
   ambiguous = _flatten(class_spec, classes)
 
   for c in classes:
-    if vm.matcher.match_from_mro(target, c, allow_compat_builtins=False):
+    if vm.matcher(None).match_from_mro(target, c, allow_compat_builtins=False):
       return True  # A definite match.
   # No matches, return result depends on whether _flatten() was
   # ambiguous.
