@@ -65,8 +65,6 @@ _PYTYPE_SINGLE_ITEMS = {
         None, 'False', ArgInfo('--attribute-variable-annotations', None), None),
     'bind_properties': Item(
         None, 'False', ArgInfo('--bind-properties', None), None),
-    'check_parameter_types': Item(
-        None, 'True', ArgInfo('--check-parameter-types', None), None),
     'preserve_union_macros': Item(
         None, 'False', ArgInfo('--preserve-union-macros', None), None),
     'use_enum_overlay': Item(
@@ -81,6 +79,8 @@ _PYTYPE_SINGLE_ITEMS = {
     'protocols': Item(None, 'False', ArgInfo('--protocols', None), None),
     'strict_import': Item(
         None, 'False', ArgInfo('--strict-import', None), None),
+    'enforce_noniterable_strings': Item(
+        None, 'False', ArgInfo('--enforce-noniterable-strings', None), None),
 }
 
 
